#!/bin/sh

# QoS related settings
. /etc/event_functions
get_qos_eventalarm_settings

echo -ne "Content-type: text/html\r\n"
echo -ne "\r\n" # end of HTTP header
echo -ne "<html>\n
          <head>\n
          <meta http-equiv='Content-Type' content='text/html; charset=utf-8' />\n
          <script type='text/javascript' src='/include/common.js'></script>\n
          </head>\n
          <body>\n"

if [ "$REQUEST_METHOD" = "POST" ]; then
	strparam=`cat $stdin | cut -c1-$CONTENT_LENGTH`
else
	strparam=$QUERY_STRING
fi


PATH=/bin:/sbin:/usr/bin:/usr/sbin
NET_STATUS=/var/run/netstatus
SMTPC="/usr/bin/smtpclient"
FTPMPUT_PROG="/usr/bin/ftpmput"
FTPS_PROG="/usr/sbin/ftps"
HTTP_CLIENT_PROG="/usr/sbin/httpclient"
CURL_PROG="/sbin/curl"
SEND_FILE="/var/run/test.txt"
SMBMOUNT="/usr/sbin/smbmount"
UMOUNT="/bin/umount"
CONF_PROG="/usr/sbin/confclient"
SMTPS_CONF="/etc/conf.d/smtps.conf"
STUNNEL_PROG="/usr/sbin/stunnel"
TEMP_FILE="/mnt/ramdisk/event_mail_tempfile"
pidfile="/var/run/smtps.pid"
pemfile="/etc/conf.d/privatekey.pem"
certfile="/etc/conf.d/publickey.cert"
CONFCLIENT="/usr/sbin/confclient"

[ -e ${CURL_PROG} ] && cURL="true"

translator()
{
    echo -e "<script type="text/javascript">document.write(translator('$1'))</script>"
}

if [ ! -f "$NET_STATUS" ]
then
	translator "please_check_internet_connection"
	echo "</body></html>"
	exit 1
fi

# Get Query string
strparam=`except_decode.sh $strparam`
type=`echo "$strparam" | sed -n 's/^.*type=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
address=`echo "$strparam" | sed -n 's/^.*address=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
username=`echo "$strparam" | sed -n 's/^.*username=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
passwd=`echo "$strparam" | sed -n 's/^.*passwd=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
port=`echo "$strparam" | sed -n 's/^.*port=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
sslmode=`echo "$strparam" | sed -n 's/^.*sslmode=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
passive=`echo "$strparam" | sed -n 's/^.*passive=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
location=`echo "$strparam" | sed -n 's/^.*location=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
senderemail=`echo "$strparam" | sed -n 's/^.*senderemail=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
recipientemail=`echo "$strparam" | sed -n 's/^.*recipientemail=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
workgroup=`echo "$strparam" | sed -n 's/^.*workgroup=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
url=`echo "$strparam" | sed -n 's/^.*url=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
groupidx=`echo "$strparam" | sed -n 's/^.*groupidx=\([^&]*\).*$/\1/p' | sed "s/%20/ /g"`
passwdflag=`echo "$strparam" | sed -n 's/^.*passwd=\([^&]*\).*$/true/p' | sed "s/%20/ /g"`
#########################################################
generate_https_necessary_file()
{
	gencert self > /dev/null
	chmod 600 /etc/conf.d/privatekey.pem
	gencert certinfo > /dev/null
}
#########################################################
check_smtp_over_https()
{
	if [ "$sslmode" = "1" ] && [ -f "$SMTPS_CONF" ]; then
		# check smtps.conf connect settings
		smtp_new_address="$address:$port"
		smtp_org_address=`grep "connect" $SMTPS_CONF | sed -n 's/connect = //p'`
		if [ "$smtp_new_address" != "$smtp_org_address" ]; then
			grep -v "connect" $SMTPS_CONF > $TEMP_FILE 
			echo "connect = $smtp_new_address" >> $TEMP_FILE
			mv $TEMP_FILE $SMTPS_CONF
		fi 
				
		if [ -f $pemfile ] && [ -f $certfile ]; then
			`$STUNNEL_PROG $SMTPS_CONF`
			if [ "$?" != "0" ];then
				generate_https_necessary_file
				`$STUNNEL_PROG $SMTPS_CONF`
			fi
		else
			generate_https_necessary_file
			`$STUNNEL_PROG $SMTPS_CONF` 
		fi
	fi
}
#########################################################
generate_test_file()
{
	# Generate file test if it doesn't exist
        if [ ! -f "$SEND_FILE" ]; then
        	echo "[NOTIFICATION]The Result of Server Test of Your IP Camera" > "$SEND_FILE"
        fi
}

#########################################################
if [ ! -n "$passwdflag" ]; then
	passwd=`$CONFCLIENT -g server_i${groupidx}_${type}_passwd -p 9 -t Value`
fi

#########################################################
HTTPS_CLIENT_CONF="../conf.d/usrdef/https_client.conf"
HTTPS_CLIENT_PID_FILE="/var/run/httpsclient.pid"

prepare_protocol_over_stunnel()
{
	CONF="$1"
	TARGET_ADDRESS="$2"
	TARGET_PORT="$3"

	tmpFile="$CONF.bak"

	if [ -f "$CONF" ]; then
		new_address="$TARGET_ADDRESS:$TARGET_PORT"
		org_address=`grep "connect" $CONF | sed -n 's/connect = //p'`
		if [ "$new_address" != "$org_address" ]; then
			grep -v "connect" $CONF > $tmpFile
			echo "connect = $new_address" >> $tmpFile
			mv $tmpFile $CONF
		fi

		if [ -f $pemfile ] && [ -f $certfile ]; then
			`$STUNNEL_PROG $CONF`
			if [ "$?" != "0" ];then
				generate_https_necessary_file
				`$STUNNEL_PROG $CONF`
			fi
		else
			generate_https_necessary_file
			`$STUNNEL_PROG $CONF`
		fi
	fi
}

#########################################################
case "$type" in
    "email")
	# set mail contents
	generate_test_file
	body="The Result of Server Test of Your IP Camera."
	title="[NOTIFICATION]The Result of Server Test of Your IP Camera"
	mime_type="-M 2"
	
	if [ -n "$username" ]; then
		auth="-U ${username} -p ${passwd}"
	else
		auth=""
	fi
			
	if [ -n "$address" ] && [ -n "$recipientemail" ]; then
		#echo "$body" | sh -c "$SMTPC -s \"$title\" $mime_type -f \"$senderemail\" -S \"$address\" -P $port $auth \"$recipientemail\"" 
		if [ "$sslmode" = "1" ]; then
			check_smtp_over_https
			$SMTPC -s "$title" $mime_type -b $SEND_FILE -f "$senderemail" -S "127.0.0.1" -P "25" $auth "$recipientemail" $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
		else
			$SMTPC -s "$title" $mime_type -b $SEND_FILE -f "$senderemail" -S "$address" -P $port $auth "$recipientemail" $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
		fi
		if [ "$?" = "0" ]
		then
            translator "the_email_has_been_sent_successfully"
		else
            translator "error_in_sending_email"
		fi
	else
        translator "please_define_mail_server_location"
	fi
	
	if [ "$sslmode" = "1" ]; then
		kill -9 `cat $pidfile`
	fi
      ;;
    
    "ftp")
    	if [ -n "$address" ]
    	then
   			
	    	if [ -n "$username" ]; then
    			auth="-u $username -p $passwd"
	    	else
    			auth=""
    		fi
    	
	    	if [ ! -n "$port" ]; then
    			port="21"
	        fi

		if [ "$passive" = "1" ]; then
			mode=""
		else
			. $NET_STATUS
			mode="-a $IP_ADDR"
		fi
		        
        	# Generate file test if it doesn't exist
        	generate_test_file
		if [ -z $location ]; then
			DIR="."
		else
			DIR="$location"
		fi

		if [ -x ${FTPS_PROG} ]; then
			${FTPS_PROG} $mode -t 60 $auth -P $port $address "$DIR" $SEND_FILE $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
		else
			${FTPMPUT_PROG} $mode -t 60 $auth -P $port $address "$DIR" $SEND_FILE $COS_PRIORITY_OPT $DSCP_PRIORITY_OPT
		fi

		if [ "$?" = "0" ] 
		then
                translator "ftp_transmission_successfully"
			echo "</body></html>"
		        exit 0
	        else
                translator "ftp_transmission_failed"
       		fi
       	else
            translator "please_define_ftp_address"
       	fi
      ;;
    
    "http" | "https")
		# Call generate test file function
		generate_test_file
		
		if [ -n "$url" ]
   		then
	   		HTTP_URL="$url"
	        # kenping, 2007/04/16, add for http client app
	        #########################################################		
	        # remove "http://"
	        case "$HTTP_URL" in
	            http://*)
					HTTP_URL=${HTTP_URL#http://}
					OVER_STUNNEL="false"
					;;
				https://*)
					HTTP_URL=${HTTP_URL#https://}
					OVER_STUNNEL="true"
					;;
			esac
	        # extract ip and port
	        TMP_IP_PORT=${HTTP_URL%%/*}
			if [ "`echo ${TMP_IP_PORT} | sed -e 's/^\(.\{1\}\).*$/\1/'`" == "[" ]; then
				if [ "`echo ${TMP_IP_PORT} | sed -e 's/\(^.*\)\(.$\)/\2/'`" == "]" ]; then
					HTTP_IP=`echo ${TMP_IP_PORT} | sed 's/^.\(.*\).$/\1/'`
					HTTP_PORT=80
				else
					TMP_IP_PORT=`echo ${TMP_IP_PORT} | sed 's/\(.*\):/\1 /'`
					HTTP_IP=`echo ${TMP_IP_PORT} | cut -d ' ' -f 1 | sed 's/^.\(.*\).$/\1/'`
					HTTP_PORT=`echo ${TMP_IP_PORT} | cut -d ' ' -f 2`
				fi

				IPv6="v6"
			else
				HTTP_IP=${TMP_IP_PORT%%:*}
				if [ "$HTTP_IP" != "$TMP_IP_PORT" ]
				then
					HTTP_PORT=${TMP_IP_PORT#$HTTP_IP:}
				else
					HTTP_PORT=80
				fi
			fi
	        # extract url
	        HTTP_URL="/${HTTP_URL#*/}"

			if [ "${OVER_STUNNEL}" = "true" ] && [ "${cURL}" != "true" ]; then
				prepare_protocol_over_stunnel ${HTTPS_CLIENT_CONF} ${HTTP_IP} ${HTTP_PORT}
				HTTP_IP=127.0.0.1
				HTTP_PORT=26
				IPv6=""
			fi
	        
	        # extract user name
			HTTP_USERNAME=$username
		    # extract password
			HTTP_PASSWD=$passwd

			if [ "${cURL}" = "true" ]; then
				[ "${IPv6}" = "v6" ] && HTTP_IP="[${HTTP_IP}]"
				http_ret_code=$("${CURL_PROG}" -s -o "/dev/null" -w "%{http_code}" "${type}://${HTTP_IP}:${HTTP_PORT}${HTTP_URL}" -u "${HTTP_USERNAME}:${HTTP_PASSWD}" -F "data=@${SEND_FILE}" -k --connect-timeout 25)
			else
				$HTTP_CLIENT_PROG -M UPLOAD -t Text -file ${SEND_FILE} -S ${HTTP_IP} ${HTTP_PORT} "${HTTP_USERNAME}" "${HTTP_PASSWD}" ${IPv6} -U "${HTTP_URL}" > /dev/null
			fi

			if [ "$?" = "0" -a "${cURL}" = "" -o "${cURL}" = "true" -a "${http_ret_code}" = "200" ]
			then
                translator "http_transmission_successfully"
				echo "</body></html>"
				
				[ -f "${HTTPS_CLIENT_PID_FILE}" ] && kill -9 "$(cat ${HTTPS_CLIENT_PID_FILE})" && rm -f ${HTTPS_CLIENT_PID_FILE}
				exit 0
			else
                translator "http_transmission_failed"
			fi
		else
            translator "please_define_a_valid_http_address"
		fi

		[ -f "${HTTPS_CLIENT_PID_FILE}" ] && kill -9 "$(cat ${HTTPS_CLIENT_PID_FILE})" && rm -f ${HTTPS_CLIENT_PID_FILE}
      ;;
        
    "ns")
	smb_path="`echo $location | sed -e 's#[\]#/#g'`"
	tmp_path=${smb_path#//}
	smb_host=${tmp_path%%/*}
	tmp_path=${smb_path#//*/}
	smb_sharefolder=${tmp_path%%/*}
	smb_childfolder=${smb_path#//$smb_host/$smb_sharefolder}
	smb_path1="//${smb_host}/${smb_sharefolder}"	

	destination_path="/mnt/auto/i99"
	if [ -n "$username" ]; then
		if [ -n "$workgroup" ]; then
			parameter="sec=ntlm,username=$username,password=$passwd,workgroup=$workgroup"
		else
			parameter="sec=ntlm,username=$username,password=$passwd"
		fi
	else
		parameter="sec=none"
	fi
	echo "i99 -fstype=cifs,$parameter :$smb_path1" >> /mnt/flash/etc/auto.conf
	cd $destination_path


	if [ "$?" = "0" ]
	then
		# Generate file test if it doesn't exist
		generate_test_file
		cp $SEND_FILE $destination_path
		if [ "$?" = "0" ]
		then 
			translator "mount_successfully"
			echo "</body></html>"
			sleep 1
			cd /mnt/ramdisk
			$UMOUNT $destination_path
			sed -i '/i99 -fstype=cifs/d' /mnt/flash/etc/auto.conf
			exit 0
		else
			translator "mount_success_but_transmit_denied"
			echo "</body></html>"
			sleep 1
			cd /mnt/ramdisk
			$UMOUNT $destination_path
			sed -i '/i99 -fstype=cifs/d' /mnt/flash/etc/auto.conf
			exit 0
		fi
	else
		translator "mount_failed"
		sed -i '/i99 -fstype=cifs/d' /mnt/flash/etc/auto.conf
	fi
	echo "</body></html>"
	exit 0
      ;;

	"sdcard")
		destination_folder="/mnt/auto/CF/test"

		if [ ! -d "$destination_folder" ]; then
			mkdir $destination_folder
		fi

		if [ -d "$destination_folder" ]; then
			generate_test_file
			cp $SEND_FILE $destination_folder

			if [ "$?" = "0" ]; then
				translator "test_sd_card_successfully"
				echo "</body></html>"
				rm $SEND_FILE
				sleep 1
				exit 0
			fi
		fi

		translator "test_sd_failed"
		translator "the_device_is_not_existed"
		translator "check_sd_card"
	;;
esac
echo "</body></html>"
exit 1

