#!/bin/sh

cmd=${1}
pkgpath=${2}
new_package=${3}
prog="tracker_app"
binpath="${pkgpath}/${prog}"
backup_db_prog="backup_db_app"
backup_db_binpath="${pkgpath}/${backup_db_prog}"
coredump_filename="/mnt/flash2/core.%e"
coredump_proc="/proc/sys/kernel/core_pattern"

vcabinarydir=$(dirname $(readlink -f $0))
report_push_path="/mnt/flash2/reportpush"
www_path="${pkgpath}/www"

vca_boa_patch="${pkgpath}/conf.d/usrdef/vca_alias.conf"
vca_boa_conf="/mnt/flash/etc/conf.d/boa/vadp-enabled/vca_alias.conf"

LOG_PATH="/mnt/flash2/logs"
LOG_FILE="${LOG_PATH}/initScript.log"

STEREO_LENS_CALIBRATE="/etc/init.d/stereolenscalibrate"

CFG_ONVIF_MEDIA_PROFILE_PATH="/etc/conf.d/config_onvif/config_onvif_media_profile.xml"
CFG_ONVIF_METADATA_PATH="/etc/conf.d/config_onvif/config_onvif_metadata.xml"
FIFO_CMD="/usr/bin/fifocmd"
FIFO_PATH="/var/run/trackerfifo"
NotifyMetadataPid="/var/run/notifymetadata.pid"
NotifyDITriggerPid="/var/run/notifyditrigger.pid"
TRACKER_PACKAGE_PATH="/mnt/flash2/tracker_package_path"
FW_VERSION="/etc/firmware-version"
is_stereo="no"

INSTALL_ERR_MSG=""
max_log_size=1000000

Coredump_Server="172.20.2.10:/Share/coredump"
Coredump_DST="/mnt/flash2/coredump/"
IS_DEBUG="no"

AuthWebsocketProg="/usr/sbin/websocketserver"
AuthWebsocketVer="${LOG_PATH}/AuthWebsocketVer.log"

_Log()
{
	if [ -z "${SCRIPT_NAME}" ]; then
		echo "${1}"
	else
		echo "<script language='javaScript'>document.write('${1}')</script><br>"
	fi
}

ExtractXML()
{
	FILE="${1}"
	NAME="${2}"
	echo `sed -n "s/.*\"${NAME}\" Value=\"\(-\{0,1\}[0-9.]*\)\".*/\1/p" ${FILE}`
}

ExtractParamDefineXML()
{
	FILE="${1}"
	NAME="${2}"
	MODE="${3}"
	INDEX="${4}"
	PATTERN="[a-zA-Z0-9.~%]*"

	if [ ${MODE} == "range" ]; then
		PARSE=`sed -n "s/.*${NAME} ${MODE}=\"\(${PATTERN}\)\".*/\1/p" ${FILE} | sed -n "s/~/ /p"`
	elif [ ${MODE} == "default" ]; then
		PARSE=`sed -n "s/.*${NAME} range=\"${PATTERN}\" ${MODE}=\"\(${PATTERN}\)\".*/\1/p" ${FILE}`
	fi

	echo ${PARSE} | awk -v i=${INDEX} '{printf "%s\n",$i}'
}

ValueCompare()
{
	TESTVAL="${1}"
	TARGETVAL="${2}"
	
	RESULT=`awk -v x=${TESTVAL} -v y=${TARGETVAL} \
			'BEGIN
			{
				if (x>y)
				{
					print "gt"
				}
				else if (x==y)
				{
					print "eq"
				}
				else if (x<y)
				{
					print "lt"
				}
				else
				{
					print "Error"
				}
			}'`
	echo ${RESULT}
}

AutoCheckConfigAE()
{
	config_ae="${pkgpath}/conf.d/usrdef/config_onvif_ae.xml"
	CamHeight=`ExtractXML ${config_ae} "CamHeight"`
	MaxObjectHeight=`ExtractXML ${config_ae} "MaxObjectHeight"`
	MinObjectHeight=`ExtractXML ${config_ae} "MinObjectHeight"`
	AlgConfidence=`ExtractXML ${config_ae} "AlgConfidence"`
	AlgMinDisparity=`ExtractXML ${config_ae} "AlgMinDisparity"`
	AlgMaxDisparity=`ExtractXML ${config_ae} "AlgMaxDisparity"`
	AlgSensitivity=`ExtractXML ${config_ae} "AlgSensitivity"`
	ZoomInFactor=`ExtractXML ${config_ae} "ZoomInFactor"`
	UpdateSec=`ExtractXML ${config_ae} "UpdateSec"`

#	echo "CamHeight=${CamHeight}"
#	echo "MaxobjH=${MaxObjectHeight}"
#	echo "MinobjH=${MinObjectHeight}"
#	echo "AlgConfidence=${AlgConfidence}"
#	echo "AlgMinDisparity=${AlgMinDisparity}"
#	echo "AlgMaxDisparity=${AlgMaxDisparity}"
#	echo "AlgSensitivity=${AlgSensitivity}"
#	echo "ZoomInFactor=${ZoomInFactor}"
#	echo "UpdateSec=${UpdateSec}"

	configParamDef="${pkgpath}/conf.d/usrdef/config_parameter_definition.xml"
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "CamHeight" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "CamHeight" "range" "2"`
	if [ "`ValueCompare ${CamHeight} ${checkLowBound}`" == "lt" ];then
	    sed -i "s/\"CamHeight\" Value=\"${CamHeight}\"/\"CamHeight\" Value=\"${checkLowBound}\"/" ${config_ae}
	elif [ "`ValueCompare ${CamHeight} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"CamHeight\" Value=\"${CamHeight}\"/\"CamHeight\" Value=\"${checkUpBound}\"/" ${config_ae}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "MaxObjectHeight" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "MaxObjectHeight" "range" "2"`
	setDefault=`ExtractParamDefineXML $configParamDef "MaxObjectHeight" "default" "1"`
	if [ "`ValueCompare ${MaxObjectHeight} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${MaxObjectHeight} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"MaxObjectHeight\" Value=\"${MaxObjectHeight}\"/\"MaxObjectHeight\" Value=\"${setDefault}\"/" ${config_ae}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "MinObjectHeight" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "MinObjectHeight" "range" "2"`
	setDefault=`ExtractParamDefineXML $configParamDef "MinObjectHeight" "default" "1"`
	if [ "`ValueCompare ${MinObjectHeight} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${MinObjectHeight} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"MinObjectHeight\" Value=\"${MinObjectHeight}\"/\"MinObjectHeight\" Value=\"${setDefault}\"/" ${config_ae}
	fi

	checkLowBound=`ExtractParamDefineXML $configParamDef "ZoomInFactor" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "ZoomInFactor" "range" "2"`
	setDefault=`ExtractParamDefineXML $configParamDef "ZoomInFactor" "default" "1"`
	if [ "`ValueCompare ${ZoomInFactor} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${ZoomInFactor} ${checkUpBound}`" == "gt" ];then
	    sed -i "s/\"ZoomInFactor\" Value=\"${ZoomInFactor}\"/\"ZoomInFactor\" Value=\"${setDefault}\"/" ${config_ae}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "AlgConfidence" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "AlgConfidence" "range" "2"`
	setDefault=`ExtractParamDefineXML $configParamDef "AlgConfidence" "default" "1"`
	if [ "`ValueCompare ${AlgConfidence} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${AlgConfidence} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"AlgConfidence\" Value=\"${AlgConfidence}\"/\"AlgConfidence\" Value=\"${setDefault}\"/" ${config_ae}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "AlgMaxDisparity" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "AlgMaxDisparity" "range" "2"`
	setDefault=`ExtractParamDefineXML $configParamDef "AlgMaxDisparity" "default" "1"`
	if [ "`ValueCompare ${AlgMaxDisparity} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${AlgMaxDisparity} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"AlgMaxDisparity\" Value=\"${AlgMaxDisparity}\"/\"AlgMaxDisparity\" Value=\"${setDefault}\"/" ${config_ae}
		AlgMaxDisparity=${setDefault}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "AlgMinDisparity" "range" "1"`
	setDefault=`ExtractParamDefineXML $configParamDef "AlgMinDisparity" "default" "1"`
	if [ "`ValueCompare ${AlgMinDisparity} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${AlgMinDisparity} ${AlgMaxDisparity}`" == "gt" ];then
		sed -i "s/\"AlgMinDisparity\" Value=\"${AlgMinDisparity}\"/\"AlgMinDisparity\" Value=\"${setDefault}\"/" ${config_ae}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "AlgSensitivity" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "AlgSensitivity" "range" "2"`
	setDefault=`ExtractParamDefineXML $configParamDef "AlgSensitivity" "default" "1"`
	if [ "`ValueCompare ${AlgSensitivity} ${checkLowBound}`" == "lt" ] || [ "`ValueCompare ${AlgSensitivity} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"AlgSensitivity\" Value=\"${AlgSensitivity}\"/\"AlgSensitivity\" Value=\"${setDefault}\"/" ${config_ae}
	fi
	
	checkLowBound=`ExtractParamDefineXML $configParamDef "UpdateSec" "range" "1"`
	checkUpBound=`ExtractParamDefineXML $configParamDef "UpdateSec" "range" "2"`
	if [ "`ValueCompare ${UpdateSec} ${checkLowBound}`" == "lt" ];then
		sed -i "s/\"UpdateSec\" Value=\"${UpdateSec}\"/\"UpdateSec\" Value=\"${checkLowBound}\"/" ${config_ae}
	elif [ "`ValueCompare ${UpdateSec} ${checkUpBound}`" == "gt" ];then
		sed -i "s/\"UpdateSec\" Value=\"${UpdateSec}\"/\"UpdateSec\" Value=\"${checkUpBound}\"/" ${config_ae}
	fi

}

_LoadCustomActions()
{
	local dir="${pkgpath}/customaction"
	local i=0
	if [ -d ${dir} ];then

		for action in `ls ${dir}`
		do
			local description="`${dir}/${action} -h`"
			if [ -z "${description}" ];then
				continue;
			fi
			confclient -p 99 -s event_customaction_i${i}_name="${action}"
			confclient -p 99 -s event_customaction_i${i}_description="${description}"
			confclient -p 99 -s event_customaction_i${i}_path="${dir}"
			cp ${dir}/${action} /mnt/flash2/vadp/customaction/event_action${i}
			i=$(($i + 1))
		done
	fi
}

_LoadDITask()
{
	local cameraModel=`confclient -g system_info_modelname -p 7 | cut -d '=' -f 2`
	
	case ${cameraModel} in
		SC8131)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8133)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-F4)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132-F4)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-F6)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132-F6)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-TL)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		AT8131-SF)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		*)
			VCA_SOCKET_FILE=trackersck
			;;
	esac
	
	DISTATUS=`cat /var/run/iod/di0_status`
	if [ $DISTATUS -eq 1 ];then
		/usr/bin/fifocmd -s /var/run/${VCA_SOCKET_FILE} -t -r ")"	> /dev/null 2>&1
	else 
		/usr/bin/fifocmd -s /var/run/${VCA_SOCKET_FILE} -t -r "*" > /dev/null 2>&1
	fi
}

ReloadVCAApp()
{
	local cameraModel=`confclient -g system_info_modelname -p 7 | cut -d '=' -f 2`
	
	case ${cameraModel} in
		SC8131)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8133)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-F4)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132-F4)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-F6)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132-F6)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-TL)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		AT8131-SF)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		*)
			VCA_SOCKET_FILE=trackersck
			;;
	esac
	/usr/bin/fifocmd -s /var/run/${VCA_SOCKET_FILE} -t -r $'\x1F'
}

_LoadAlarmRule()
{
	local re="${pkgpath}/conf.d/usrdef/config_onvif_re.xml"
	local target="${pkgpath}/conf.d/default/config_event_description.xml"

	prog="${pkgpath}/cgi-bin/genEventDesc.cgi"

	${prog} -r ${re} -o ${target}
	/etc/init.d/onvifeventd reload
}

get_vadplinkname()
{
	idx=`readlink -f $0 | cut -d'/' -f5`
	mod_name=`tinyxmlparser -x /vadp/name -f $vcabinarydir/vadp.xml`
	link_name=`echo "${mod_name}" | sed -e 's/[[:blank:]]/-/g'`
	echo ${link_name}
}

modify_mvaasconf()
{
	sed -i "s#HTTPExtraInfoUrl \/cgi-bin\/viewer\/getparam.cgi?system_hostname&system_location_groupid&system_location_deviceid#HTTPExtraInfoUrl \/`get_vadplinkname`\/cgi-bin\/get_mvaas_param.cgi#" /etc/conf.d/mvaasd.conf
}

clear_refererconf()
{
	MAX_REFERER_COUNT=10
	server_ip=`confclient -p 99 -g system_mvaas_registerserver_address | cut -d "=" -f2`
	cnt=0
	while [ $cnt -lt $MAX_REFERER_COUNT ];do
		name=`tinyxmlparser -x /root/security/referer/RcExceptionHost/i$cnt/name -f /etc/conf.d/config_referer.xml`
		if [ $name == $server_ip ];then
			sed -i -e '/<'i$cnt'>/,/<\/'i$cnt'>/ s|<name>.*</name>|<name></name>|g' /etc/conf.d/config_referer.xml
			kill -31 `cat /var/run/boa.pid` 	#reload referer
			break;
		fi
		cnt=$((cnt+1))
	done
}

UpdateHttpPrivilege()
{
	local pkgPrivilege=`cat ${1}/info/pkg_privilege`

	if [ ${pkgPrivilege} == "viewer" ];then
		"${pkgpath}/scripts/updateBoaProtTracker" ${pkgpath}
		/etc/init.d/httpd reload
	fi
}

modify_vcaboaconf()
{
	idx=`readlink -f $0 | cut -d '/' -f5`
	cp $vca_boa_patch $vca_boa_conf
	echo -e "ScriptAlias@vadp$idx /VCA/cgi-bin/ $vcabinarydir/cgi-bin/\nAlias@vadp$idx /VCA/www/ $vcabinarydir/www/" >> $vca_boa_conf
	echo -e "ProtPath viewer_server $vcabinarydir/cgi-bin/" >> $vca_boa_conf
    echo -e "ProtPath viewer_server /mnt/flash2/vcaresource/" >> $vca_boa_conf
	echo -e "ProtPath viewer_server $vcabinarydir/www/" >> $vca_boa_conf
	/etc/init.d/httpd reload
}

remove_vcaboaconf()
{
	 rm $vca_boa_conf
	 /etc/init.d/httpd reload
}

eventConfigDI()
{
	local cmd=$1
	local file="${pkgpath}/conf.d/usrdef/di_task.xml"
	local target="/etc/conf.d/event/task/di_task.xml"
	
	case ${cmd} in
			install)
					cp ${file} ${target}
					sed -i "s#\/mnt\/flash2\/vadp-preload\/0#$vcabinarydir#" ${target}
					sed -i "s#\/mnt\/flash2\/vadp-preload\/0#$vcabinarydir#" "${pkgpath}/scripts/setDIevent"
					;;
			uninstall)
					rm ${target}
					;;
			*)
					echo "[eventConfigDI] Not support cmd = ${cmd}"
					return
					;;
	esac
	
	/etc/init.d/eventmgr reload
}

EnableRemoteCoreDump()
{
	mkdir ${Coredump_DST}

	local CAMERA_IP=`confclient -p 99 -g network_ipaddress | cut -d '=' -f2`
	local GIT_SHA=`cat ${pkgpath}/info/git_sha`
	local PLATFORM=`cat ${pkgpath}/info/pkg_platform`
	local Corefile_Prefix="core-"${CAMERA_IP}"-"${GIT_SHA}"-"${PLATFORM}
	coredump_filename=${Coredump_DST}${Corefile_Prefix}"-%e"

	umount ${Coredump_DST}
	mount -t nfs ${Coredump_Server} ${Coredump_DST} -o tcp,nolock
	if [ $? -eq 0 ];then
		coredump_args="-c unlimited"
	else
		coredump_args="-c 8192"
	fi

	ulimit ${coredump_args}
	echo "${coredump_filename}" > ${coredump_proc}
	echo ${Corefile_Prefix} > ${pkgpath}"/info/corefile_prefix"
}

EnableCoreDump()
{
	local cameraModel=`confclient -g system_info_modelname -p 7 | cut -d '=' -f 2`

	case ${cameraModel} in
		SC8131)
			coredump_args="-c 8192"
			;;
		SC8132)
			coredump_args="-c 8192"
			;;
		SC8133)
			coredump_args="-c 8192"
			;;
		SC8131-F4)
			coredump_args="-c 8192"
			;;
		SC8132-F4)
			coredump_args="-c 8192"
			;;
		SC8131-F6)
			coredump_args="-c 8192"
			;;
		SC8132-F6)
			coredump_args="-c 8192"
			;;
		SC8131-TL)
			coredump_args="-c 8192"
			;;
		AT8131-SF)
			coredump_args="-c 8192"
			;;
		*)
			coredump_args="-c 4096"
			;;
	esac

	ulimit ${coredump_args}
	echo "${coredump_filename}" > ${coredump_proc}
}

reportpushFile()
{
	pkgConfPath="${pkgpath}/conf.d"
	newPkgConfPath="${new_package}/conf.d"
	defaultFolder="default"
	usrdefFolder="usrdef"
	pkgCgiPath="${pkgpath}/cgi-bin"
	triggerReport="trigger_report"
	tinyXML="tinyxmlparser"
	updateBackupFile="scripts/updateReport"

	etcCdfPath="/etc/conf.d/CDF"
	etcEventTaskPath="/etc/conf.d/event/task"

	reportpushConfig="config_reportpush.xml"
	reportpushCDF="CDF_reportpush.xml"
	reportpushEventTask="report_task.xml"
	reportpushHttpsConfig="https_client.conf"

	cmd=$1

	case ${cmd} in
		install)
			cp "${pkgConfPath}/${defaultFolder}/${reportpushEventTask}" "${etcEventTaskPath}/${reportpushEventTask}"
			sed -i "s#\/usr\/bin#${pkgpath}\/cgi-bin#" "${etcEventTaskPath}/${reportpushEventTask}"

			cp "${pkgConfPath}/${defaultFolder}/${reportpushHttpsConfig}" "${pkgConfPath}/${usrdefFolder}/${reportpushHttpsConfig}"

			reloadEventMgr="true"
			;;
		upgrade)
			if [ -f "${pkgConfPath}/${usrdefFolder}/${reportpushConfig}" ] && [ "$(grep "${tinyXML}" "${pkgCgiPath}/${triggerReport}")" = "" ]; then
				cp "${pkgConfPath}/${usrdefFolder}/${reportpushConfig}" "${newPkgConfPath}/${usrdefFolder}/${reportpushConfig}"
				[ -f "${etcCdfPath}/${reportpushCDF}" ] && rm "${etcCdfPath}/${reportpushCDF}" && restartCfgMgr="true"
				"${new_package}/${updateBackupFile}" checkBackup
			fi
			;;
		uninstall)
			[ -f "${etcEventTaskPath}/${reportpushEventTask}" ] && rm "${etcEventTaskPath}/${reportpushEventTask}" && reloadEventMgr="true"
			"${pkgpath}/${updateBackupFile}" deleteAll
			;;
		restore)
			"${pkgpath}/${updateBackupFile}" deleteAll
			;;
		convert)
			while [ -f "${pkgConfPath}/${usrdefFolder}/${reportpushConfig}" ]
			do
				ReloadVCAApp
				sleep 3
			done
			;;
		*)
			echo "[$0] Not support cmd = ${cmd}"
			return
			;;
	esac

	[ "${restartCfgMgr}" = "true" ] && configerRestartAndWaitInitDone
	[ "${reloadEventMgr}" = "true" ] && /etc/init.d/eventmgr reload
}

KeepVCAConfig()
{
	local oldPath=${pkgpath}"/conf.d/vca"
	local newPath=${new_package}"/conf.d/vca"
	
	# Copy new package functions file
	[ "$(ls -A ${oldPath})" ] && cp -rf "${newPath}/config_vca_functions.json" "${oldPath}"
	
	rm -rf "${newPath}"
	[ "$(ls -A ${oldPath})" ] || return 0

	# Copy config file from ${oldPath} to ${newPath}.
	cp -rf "${oldPath}" "${newPath}"

	return 0
}

clearSqliteCmd()
{
	rm ${report_push_path}/*.sql
}

checkReportPushFolder()
{
	sqlite_cmd_num=`ls ${report_push_path}/*.sql | wc -l`

	if [ ${sqlite_cmd_num} -gt 0 ]; then
		clearSqliteCmd
	fi
}

checkIsStereoCam()
{	
	local cameraModel=`confclient -g system_info_modelname -p 7 | cut -d '=' -f 2`
	
	if [ ${cameraModel} == "SC8131" ] || [ ${cameraModel} == "SC8132" ] || [ ${cameraModel} == "SC8133" ] || [ ${cameraModel} == "SC8131-F4" ] || [ ${cameraModel} == "SC8132-F4" ] || [ ${cameraModel} == "SC8131-F6" ] || [ ${cameraModel} == "SC8132-F6" ] || [ ${cameraModel} == "SC8131-TL" ] || [ ${cameraModel} == "AT8131-SF" ];then
		is_stereo="yes";
	fi
}

createWebLayoutSoftLink()
{
	local cameraModel=${1}
	local webLayoutName="layout"
	local targetWebLayoutName="${webLayoutName}_${cameraModel}"

	local webLayoutFilePath="${www_path}/${webLayoutName}.xml"
	local targetWebLayoutFilePath="${www_path}/${targetWebLayoutName}.xml"

	if [ -e "${webLayoutFilePath}" ]; then
		rm -f ${webLayoutFilePath}
	fi

	ln -s ${targetWebLayoutFilePath} ${webLayoutFilePath}
}

checkWebLayoutByModel()
{
	local cameraModel=`confclient -g system_info_modelname -p 7 | cut -d '=' -f 2`

	case ${cameraModel} in
		SC8131)
			createWebLayoutSoftLink "SC8131"
			;;
		SC8132)
			createWebLayoutSoftLink "SC8132"
			;;
		SC8133)
			createWebLayoutSoftLink "SC8133"
			;;
		SC8131-F4)
			createWebLayoutSoftLink "SC8131"
			;;
		SC8132-F4)
			createWebLayoutSoftLink "SC8132"
			;;
		SC8131-F6)
			createWebLayoutSoftLink "SC8131"
			;;
		SC8132-F6)
			createWebLayoutSoftLink "SC8132"
			;;
		SC8131-TL)
			createWebLayoutSoftLink "SC8131"
			;;
		AT8131-SF)
			createWebLayoutSoftLink "SC8131"
			;;
	esac
}

configerRestartAndWaitInitDone()
{
	[ ! -d ${LOG_PATH} ] && mkdir -p ${LOG_PATH}

	configMgr="/etc/init.d/configer"
	confClient="/usr/sbin/confclient"
	configMgrPidFile="/var/run/configer.pid"

	${configMgr} restart

	sleep 1

	errTry=3

	count=0
	while [ ! -e "${configMgrPidFile}" ]
	do
		${configMgr} restart
		sleep 1

		count=$(( count + 1 ))
		if [ ${count} -gt ${errTry} ]; then
			echo "$(date) : configerRestartAndWaitInitDone restart configer failed, try count = ${count}" >> ${LOG_FILE}
			return
		fi
	done

	count=0
	until ${confClient} -g test -p 7 > /dev/null
	do
		sleep 1
		count=$(( count + 1 ))
		if [ ${count} -gt ${errTry} ]; then
			echo "$(date) : configerRestartAndWaitInitDone failed, try count = ${count}" >> ${LOG_FILE}
			break
		fi
	done
}

StartTrackAppByCPU()
{
	local cameraCPU=`cat /proc/umap/ive | grep Version | awk '{print substr($3,match($3,/Hi/),length($3));}' | cut -d"_" -f1`

	case ${cameraCPU} in
		Hi3519V101)
			start-stop-daemon --start --quiet --background --name $prog --exec taskset 0x02 $binpath
			;;
		Hi3516A)
			cd ${pkgpath}
			start-stop-daemon --start --quiet --background --name $prog --exec $binpath
			;;
		*)
			start-stop-daemon --start --quiet --background --name $prog --exec $binpath ${pkgpath}
			;;
	esac
}

CreateSoftLinkSSMUX()
{
        local supportSsmuxVersions='1.1.1.0 1.0.0.7'

        for version in $supportSsmuxVersions; do
                local ssmuxPath="/www/include/ssmux-$version.js"
                if [ -e $ssmuxPath ]; then
                        break;
                fi
        done

        local destinationSsmuxPath="${www_path}/ssmux.js"
        if [ -e "${destinationSsmuxPath}" ]; then
                rm -f ${destinationSsmuxPath}
        fi
        ln -s ${ssmuxPath} ${destinationSsmuxPath}
}

EnableMetadata()
{
	checkIsStereoCam

	if [ ${is_stereo} == "no" ];then	
		local analyticsTagValue="VVTK_STRACKER"
		local metadataTagValue="metadata0"
		local mediaProfileNum=`cat /etc/conf.d/config_onvif/config_onvif_media_profile.xml | grep "/videoanalyticsconfiguration" | wc -l`

		local Index=0
		while [ "$Index" -lt "$mediaProfileNum" ];
		do
			tinyxmlparser -f $CFG_ONVIF_MEDIA_PROFILE_PATH -x /mediaconf/profile/i$Index/videoanalyticsconfiguration/token -w $analyticsTagValue
			tinyxmlparser -f $CFG_ONVIF_MEDIA_PROFILE_PATH -x /mediaconf/profile/i$Index/metadataconfiguration/token -w $metadataTagValue

			Index=$(($Index+1))
		done
		
		tinyxmlparser -f $CFG_ONVIF_METADATA_PATH -x /root/metadataconfiguration/AnalyticsEngineConfiguration -w $analyticsTagValue
	fi
}

EnableEmulationLib()
{	
	checkIsStereoCam
	local emulationLibPath="${pkgpath}/lib/"

	if [ ${is_stereo} == "no" ] && [ -d ${emulationLibPath} ] ;then
		export LD_LIBRARY_PATH=${emulationLibPath}
	fi
}

RemoveEventTriggerList()
{
	TriggerListNum=`confclient -p 99 -t Value -g vadp_event_ntrigger`

	i=0
	while [ $i -le ${TriggerListNum} ]
	do
		TriggerTopic=`confclient -p 99 -t Value -g vadp_event_triggerlist_i${i}_topic`
		case "${TriggerTopic}" in
			tns1:RuleEngine/TrackerAlarm/* )
				echo remove ${TriggerTopic} >> $LOG_FILE
				confclient -p 99 -s vadp_event_triggerlist_i${i}_topic=
				;;
		esac
		i=$(($i+1))
	done
}

validation()
{
	[ -e "${pkgpath}/scripts/validation.sh" ] || return

	"${pkgpath}"/scripts/validation.sh "${1}"
}

SetupONVIFPropertyEvent()
{
	if [ -d "${pkgpath}/conf.d/default/PEvent" ]; then
		cp -rf ${pkgpath}/conf.d/default/PEvent/* /mnt/ramdisk/event
	fi
}

NotifyMetadataRecord()
{
	local recordstatus=`confclient -p 99 -g recording_i0_enable | cut -d '=' -f 2`

	while [ "$(lsof "${FIFO_PATH}")" == "" ]
	do
		sleep 1
	done

	if [ $recordstatus == "1" ];then
		${FIFO_CMD} -p "${FIFO_PATH}" -r $'\x2D'
	fi
}

NotifyDITrigger()
{
	local distatus=`cat /var/run/iod/di0_status`
	local cameraModel=`confclient -g system_info_modelname -p 7 | cut -d '=' -f 2`

	case ${cameraModel} in
		SC8131)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8133)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-F4)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132-F4)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-F6)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8132-F6)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		SC8131-TL)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		AT8131-SF)
			VCA_SOCKET_FILE=stereotrackersck
			;;
		*)
			VCA_SOCKET_FILE=trackersck
			;;
	esac

	SCK_PATH="/var/run/${VCA_SOCKET_FILE}"
	while [ "$(lsof "${SCK_PATH}")" == "" ]
	do
		sleep 1
	done

	if [ $distatus == "1" ];then
		$vcabinarydir/bin/trackersckcmd -c DI_0_ON > /dev/null 2>&1
	else
		$vcabinarydir/bin/trackersckcmd -c DI_0_OFF > /dev/null 2>&1
	fi
}

CheckPlatform()
{
	# Check platform information
	local check_package=$1
	local camPlatform=`${check_package}/scripts/getCameraPlatform`
	local pkgPlatform=`cat ${check_package}/info/pkg_platform`

	if [ ${camPlatform} != ${pkgPlatform} ];then
		INSTALL_ERR_MSG="Platform mismatch, platform is ${camPlatform} and package support on ${pkgPlatform}"
	fi
}

CheckBrand()
{
	local pkgBrand=`cat ${check_package}/info/pkg_brand`
	local camBrand=""

	local isOBM=`cat ${FW_VERSION} | grep -c "VVTK"`
	if [ ${isOBM} == "1" ];then
		camBrand="obm"
	else
		camBrand="odm"
	fi

	if [ ${camBrand} == "odm" ] && [ ${pkgBrand} == "obm" ];then
		INSTALL_ERR_MSG="This package only support VIVOTEK"
	elif [ ${camBrand} == "obm" ] && [ ${pkgBrand} == "odm" ];then
		INSTALL_ERR_MSG="This package only support ODM"
	fi
}

CheckInstall()
{
	local check_package=$1
	CheckPlatform ${check_package}
	CheckBrand
}

RecordPackagePath()
{
 	echo ${pkgpath} > ${TRACKER_PACKAGE_PATH}
}

CheckDebug()
{
	if [ -d "${pkgpath}/debug/" ]; then
		IS_DEBUG="yes"
	else
		IS_DEBUG="no"
	fi
}

GetVcaPkgIndex()
{
	c=0
	while [ $c -lt 10 ]
	do
		name=`confclient -p 99 -g vadp_module_i"$c"_name | cut -d'=' -f2`
		if [ "${name}" == "Stereo Counting" ] || [ "${name}" == "Counting" ] || [ "${name}" == "VCA" ];then
			echo $c
			break
		fi
		c=$(($c+1))
	done
}

StopApp()
{
	start-stop-daemon -K --signal 15 --quiet --name $prog
	start-stop-daemon -K --signal 15 --quiet --name $backup_db_prog
}

RestoreVcaCfg()
{
    confFolder="${pkgpath}/conf.d"
    confVca="${confFolder}/vca"
    confDefault="${confFolder}/default"
    confUsrDef="${confFolder}/usrdef"

    mv -f "${confVca}/config_vca_functions.json" "${confDefault}/config_vca_functions.json"
    rm -rf "${confVca}"/*
    mv -f "${confDefault}/config_vca_functions.json" "${confVca}/config_vca_functions.json"
    cp "${confDefault}/config_onvif_ae.xml" "${confUsrDef}/config_onvif_ae.xml"
    cp "${confDefault}/config_onvif_re.xml" "${confUsrDef}/config_onvif_re.xml"
}

GetAuthWebsocketVer()
{
	${AuthWebsocketProg} -h | grep Version | awk '{print $5}' > ${AuthWebsocketVer}
}

CheckDebug
touch $LOG_FILE
log_size=$(wc -c <$LOG_FILE)
if [ $log_size -gt $max_log_size ]; then
	rm $LOG_FILE
fi
echo "`date +"%Y-%m-%d %T"` : ${cmd},${pkgpath},${new_package},${binpath},${backup_db_binpath}" >> $LOG_FILE

case ${cmd} in
	start)
		if test $(ps | grep -v grep | grep /$prog | wc -l) -gt 0 -o $(ps | grep -v grep | grep /$backup_db_prog | wc -l) -gt 0
		then
			echo 'tracker_app or backup_db_app is still running' >> $LOG_FILE
			PkgIndex=`GetVcaPkgIndex`
			confclient -s vadp_module_i${PkgIndex}_status="off" -p 7
			StopApp
		else
			echo 'tracker_app or backup_db_app is not running'
            if [ -x "${pkgpath}/bin/ldconfig" ];then
			    export LD_LIBRARY_PATH="${pkgpath}/lib/"
    			${pkgpath}/bin/ldconfig -C /etc/ld.so.cache -n ${pkgpath}/lib
            fi
			UpdateHttpPrivilege ${pkgpath}
			_LoadAlarmRule
			_LoadDITask
			_LoadCustomActions
			EnableEmulationLib
			modify_vcaboaconf
			GetAuthWebsocketVer
            
			if [ ${IS_DEBUG} == "yes" ];then
				EnableRemoteCoreDump
			else
				EnableCoreDump
			fi

			EnableMetadata
			CreateSoftLinkSSMUX
			checkWebLayoutByModel
			StartTrackAppByCPU
			start-stop-daemon --start --quiet --background --name $backup_db_prog --exec $backup_db_binpath
			${STEREO_LENS_CALIBRATE} restart
			reportpushFile convert
			validation auto
			SetupONVIFPropertyEvent
			NotifyMetadataRecord & echo $! > ${NotifyMetadataPid}
			NotifyDITrigger & echo $! > ${NotifyDITriggerPid}
			PkgIndex=`GetVcaPkgIndex`
			confclient -s vadp_module_i${PkgIndex}_status="on" -p 7
		fi
		;;
	stop)
		# killall ${prog}
		validation stop
		StopApp
		;;
	install)
		CheckInstall ${pkgpath}
		if [ "${INSTALL_ERR_MSG}" != "" ];then
			_Log "${INSTALL_ERR_MSG}"
			rm -rf ${pkgpath}
			exit 1
		fi
		
		modify_mvaasconf
		modify_vcaboaconf
		eventConfigDI install
		reportpushFile install
		# For genReport not clear sqlite command files issue, user can upgrade or restore FW to fix it.
		checkReportPushFolder
		RecordPackagePath
		;;
	remove)
		clear_refererconf
		remove_vcaboaconf
		RemoveEventTriggerList
		eventConfigDI uninstall
		reportpushFile uninstall
		;;
	restore)
        RestoreVcaCfg
        reportpushFile restore
		;;
	upgrade)
		# Optional. Doing upgrade
		_Log "Upgrading Tracker."

		CheckInstall ${new_package}
		if [ "${INSTALL_ERR_MSG}" != "" ];then
			_Log "${INSTALL_ERR_MSG}"
			exit 1
		fi

        /usr/sbin/onvifcfg_configer -b ${pkgpath}/conf.d/usrdef/ -n ${new_package}/conf.d/usrdef/
        reportpushFile upgrade
        KeepVCAConfig

		rm -rf ${pkgpath}
		mv ${new_package} ${pkgpath}

		confclient -s capability_tampering=1 -p 99 > /dev/null
		RecordPackagePath
		;;
	reload)
		_LoadAlarmRule
		_LoadDITask
		_LoadCustomActions
		ReloadVCAApp
		validation auto
		;;
	*)
		;;
esac

exit 0

