#!/bin/sh
taskPattern="<?xml version=\"1.0\" encoding=\"ISO-8859-1\"?><eventmgr version=\"0102\"><maxprocess>2</maxprocess>
<schedule id=\"0\">
<duration>
<weekday>%s</weekday>
<time>%s</time>
</duration>
</schedule>
<event id=\"0\">
<description>start validation</description>
<scheduleno>0</scheduleno>
<delay>-2</delay>
<priority>1</priority>
<trylater>0</trylater>
<process>%s start</process>
</event>
<event id=\"1\">
<description>stop validation</description>
<scheduleno>!0</scheduleno>
<delay>-2</delay>
<priority>1</priority>
<trylater>0</trylater>
<process>%s stop</process>
</event>
</eventmgr>"

ACTION="${1}"

if [ -L $0 ] ; then
	BASEDIR=$(cd "$(dirname "$(readlink $0)")"; pwd -P) # for symbolic link
else
	BASEDIR=$(cd "$(dirname "$0")"; pwd -P) # for normal file
fi

FIFOCMD="/usr/bin/fifocmd"
RECORDER_FIFO="/tmp/recorderfifo"
VENC_CTRL_FIFO="/var/run/venc/c0/controlfifo"
STORAGE="/mnt/ramdisk/validation"
TINY_XML="/usr/bin/tinyxmlparser"
VALIDATION_CFG="${BASEDIR}/../conf.d/vca/config_validation.xml"
EVENT_MGR="/etc/init.d/eventmgr"
CONFCLIENT="/usr/sbin/confclient"
VALIDATION_PID_FILE="/var/run/validation.pid"
BACKUP_PATH="/mnt/flash2/validation"
FTP_STATUS="1"
RECORDER_PID_FILE="/var/run/recorder.pid"

ATTR_FREESIZE_LINE=""
VDO_TARGET_SIZE=1536
# start/stop condition
STORAGE_MAX_SIZE=20480
STORAGE_AVAILABLE_SIZE=`expr ${STORAGE_MAX_SIZE} / 3`
#STOP_SIZE=23040  # VDO_TARGET_SIZE * 15
#START_SIZE=30720 # VDO_TARGET_SIZE * 20
FTP_TIMEOUT=30
VALIDATION_POLLING=0.5
WATCH_RAMDISK_POLLING=0.2
RETURN_STRING="/bin/echo"
log_file=$(sed -n 1p /etc/syslog.conf | cut -d ' ' -f 2)

[ ! -d "${BACKUP_PATH}" ] && mkdir -p "${BACKUP_PATH}"

logToSys()
{
	echo "$(date +"%b") $(date +"%e") $(date +"%k:%M:%S")  [Validation]: $1" >> ${log_file}
}

getXmlItemVal()
{
	xmlPath=$1

	${TINY_XML} -x "${xmlPath}" -f "${VALIDATION_CFG}"
}

reSentBackup()
{
	oldestFile=$(ls -tr ${BACKUP_PATH} | head -1)

	[ "${oldestFile}" != "" ] && ftpSender "${BACKUP_PATH}/${oldestFile}"
}

startRecord()
{
	${FIFOCMD} -p /var/run/trackerfifo -r $'\x2D' > /dev/null 2> /dev/null
	lastStream="$(${CONFCLIENT} -g capability_nmediastream -p 7 -t Value)"
	echo "man-record-start name=validation videosocket=/var/run/venc/c0/s$((${lastStream}-1))/mediasck_2 metadatasocket=/var/run/venc/c0/metasck length=60 maxsize=${VDO_TARGET_SIZE} location=${STORAGE} totalsize=1 mediaoption=1" > ${RECORDER_FIFO}
	${FIFOCMD} -n -p ${VENC_CTRL_FIFO} -c "<control id=\"2\" stream=\"${lastStream}\"><forceCI/><output>start</output></control>" > /dev/null 2> /dev/null
	sleep 1
}

stopRecord()
{
	if [ "$(pgrep "recorder")" != "" ]; then
		echo "man-record-stop name=validation" > ${RECORDER_FIFO}
		sleep 1
	else
		logToSys "Recorder not exist, so skip sending stop command to recorder"
	fi
}

setRecordEn()
{
	action="${1}"
	record_en="${2}"

	case "${action}" in
		"start")
			if [ "${record_en}" = "no" ] && [ -e "${VALIDATION_PID_FILE}" ]; then
				startRecord
				record_en="yes"
			fi
			;;
		"stop")
			if [ ${record_en} = "yes" ]; then
				stopRecord
				record_en="no"
			fi
			;;
		"restart")
			if [ -e "${VALIDATION_PID_FILE}" ]; then
				startRecord
				record_en="yes"
			fi
			;;
		*)
			;;
	esac

	${RETURN_STRING} "${record_en}"
}

backupToFlash()
{
	backupFile="$1"
	totalMaxSize="${STORAGE_MAX_SIZE}"

	mv "${backupFile}" "${BACKUP_PATH}/$(basename "${backupFile}")"
	sync

	curTotalSize=$(du -k "${BACKUP_PATH}" | awk '{print $1}')

	diffSize=$((totalMaxSize - curTotalSize))

	if [ ${diffSize} -lt 0 ]; then
		for backFile in $(ls -tr "${BACKUP_PATH}")
		do
			diffSize=$((diffSize + $(du -k "${BACKUP_PATH}/${backFile}" | awk '{print $1}')))
			logToSys "flash is almost full, remove file ${backFile}. check FTP connection"
			rm "${BACKUP_PATH}/${backFile}"

			[ ${diffSize} -gt 0 ] && break
		done
	fi
}

ftpSender()
{
	pathName="$1"
	ftpmputProg="/usr/bin/ftpmput"
	ftpsProg="/usr/sbin/ftps"

	[ -e "${pathName}" ] || return

	checkMaster="$(wget -q -O - http://127.0.0.1/VCA/Stitching/StitchedMapInfo | awk 'BEGIN {FS="TranslationX2DToMaster"}; {print $1}' | cut -d ',' -f 10 | cut -d ':' -f 2 | sed s/\"//g)"
	if [ ${checkMaster} = "stitched" ]; then
		MASTER="M_"
	fi

	if [ "$(echo "${pathName}" | grep "${STORAGE}")" != "" ]; then
		createTime=$(stat -c %x "${pathName}")
		systemInfo="${MASTER}$(${CONFCLIENT} -g system_info_serialnumber -p 7 -t Value)"
		composeName="${STORAGE}/${systemInfo}_$(echo "${createTime}" | awk '{print $1}')_$(echo "${createTime}" | awk '{print $2}' | cut -d '.' -f 1 | sed s/://g).mp4"
		mv "${pathName}" "${composeName}"
		needBackup="yes"
	else
		composeName="${pathName}"
	fi

	user="$(getXmlItemVal /validation/ftp/user)"
	pwd="$(getXmlItemVal /validation/ftp/pwd)"
	port="$(getXmlItemVal /validation/ftp/port)"
	address="$(getXmlItemVal /validation/ftp/address)"
	folder="$(getXmlItemVal /validation/ftp/folder)"

	if [ -z "${folder}" ]; then
		folder="/"
	fi

	if [ -x ${ftpsProg} ]; then
		${ftpsProg} -u "${user}" -p "${pwd}" -t "${FTP_TIMEOUT}" -P "${port}" "${address}" "${folder}" "${composeName}"
	else
		${ftpmputProg} -u "${user}" -p "${pwd}" -t "${FTP_TIMEOUT}" -P "${port}" "${address}" "${folder}" "${composeName}"
	fi

	if [ "$?" = "0"	]; then
		rm -f "${composeName}"
		FTP_STATUS="0"
	else
		[ "${needBackup}" = "yes" ] && backupToFlash "${composeName}"
		FTP_STATUS="1"
	fi
}

getFreesizeLine()
{
	attr=`df -k ${STORAGE} | sed -n '1p'`

	line=0
	for i in ${attr}
	do
		line=`expr ${line} + 1`
		if [ ${i} = "Available" ]; then
			ATTR_FREESIZE_LINE=${line}
			break;
		fi
	done
}

checkRamdisk()
{
	record_en="${1}"

	cur_size=`du -k ${STORAGE} | awk '{print $1}'`
	if [ -z "${cur_size}" ]; then
		${RETURN_STRING} "${record_en}"
	fi

	if [ ${cur_size} -gt ${STORAGE_MAX_SIZE} ] && [ "${record_en}" = "yes" ]; then
		logToSys "Validation's storage is almost full. Temporarily stop video recording"
		record_en=$(setRecordEn "stop" "${record_en}")
	elif [ ${cur_size} -lt ${STORAGE_AVAILABLE_SIZE} ] && [ "${record_en}" = "no" ]; then
		logToSys "Ramdisk is available now. Start video recording"
		record_en=$(setRecordEn "start" "${record_en}")
	fi

	${RETURN_STRING} "${record_en}"
}

checkFileReady()
{
	send_all="${1}"

	for pathName in ${STORAGE}/*.mp4
	do
		[ -e "${pathName}" ] || continue

		if [ "$(echo "${pathName}" | grep ".mp4")" != "" ] && [ "$(lsof "${pathName}")" = "" ]; then
			ftpSender "${pathName}"
		fi

		[ "${send_all}" = "no" ] && break
	done

	if [ "${FTP_STATUS}" = "0" ]; then
		reSentBackup
	fi
}

validationLoop()
{
	validation_pid="$(cat "${VALIDATION_PID_FILE}")"

	while [ -e "${VALIDATION_PID_FILE}" ]
	do
		checkFileReady "no"
		sleep ${VALIDATION_POLLING}

		cur_pid="$(cat "${VALIDATION_PID_FILE}")"
		#echo "Val check ORI:${validation_pid}, NEW: ${cur_pid}"
		if [ "${validation_pid}" != "${cur_pid}" ]; then
			logToSys "Validation PID was changed"
			#echo "ValidationLoop exit ORI:${validation_pid}, NEW: ${cur_pid}"
			break
		fi
	done

	checkFileReady "yes"
	echo "##EXIT Validation: ${validation_pid}"
}

watchRamdiskLoop()
{
	validation_pid="$(cat "${VALIDATION_PID_FILE}")"
	record_pid="$(cat "${RECORDER_PID_FILE}")"
	record_en="no"

	record_en=$(setRecordEn "start" "${record_en}")

	count=0

	while [ -e "${VALIDATION_PID_FILE}" ]
	do
		# check recorder
		if [ "$(cat "${RECORDER_PID_FILE}")" != "${record_pid}" ]; then
			logToSys "Recorder PID was changed"
			# update new pid
			record_pid="$(cat "${RECORDER_PID_FILE}")"
			record_en=$(setRecordEn "restart" "${record_en}")
		fi
		##

		# check ramdisk
		record_en=$(checkRamdisk "${record_en}")
		##

		# check validation pid
		cur_pid="$(cat "${VALIDATION_PID_FILE}")"
		#echo "Wat check ORI:${validation_pid}, NEW: ${cur_pid}"
		if [ "${validation_pid}" != "${cur_pid}" ]; then
			#echo "WatchLoop exit ORI:${validation_pid}, NEW: ${cur_pid}"
			break
		fi
		##

		# check if record was stopped in unexpected
		list=`ls ${STORAGE}`
		if [ -z "${list}" ]; then
			count=`expr ${count} + 1`
			sleep 1
		else
			count=0
		fi

		if [ ${count} -gt 30 ]; then
			logToSys "Recording was stopped unexpectedly, restart it"
			record_pid="$(cat "${RECORDER_PID_FILE}")"
			record_en=$(setRecordEn "restart" "${record_en}")
		fi
		##

		sleep ${WATCH_RAMDISK_POLLING}
	done

	record_en=$(setRecordEn "stop" "${record_en}")

	echo "##EXIT Watch: ${validation_pid}"	
}

checkDuplicated()
{
	max=${1}
	count=0
	dup="yes"

	while [ "${dup}" = "yes" ]
	do
		# check
		cur_pid=$$
		list=`ps | grep validation | awk '{print $1}' | tr "\n" " "`
		#echo "list: ${list}"
		#echo "cur_pid: ${cur_pid}"
		sleep 1

		dup="no"
		for pid in ${list}
		do
			kill -0 ${pid} > /dev/null 2> /dev/null
			retval=$?
			if [ ${retval} -eq 0 ] && [ ${pid} -ne ${cur_pid} ]; then
				dup="yes"
				break
			fi
		done
		##

		#echo "Wait duplicated process: dup=${dup}/${count}"

		count=`expr ${count} + 1`
		[ ${count} -gt ${max} ] && break
	done
}

startValidation()
{
	if [ ! -e "${VALIDATION_PID_FILE}" ]; then
		checkDuplicated "300"
		sleep 1

		# check again
		sync
		if [ ! -e "${VALIDATION_PID_FILE}" ]; then
			validationLoop & echo $! > ${VALIDATION_PID_FILE}
			watchRamdiskLoop &
		fi
		##
	else
		logToSys "Validation has started already"
		exit 0
	fi
}

stopValidation()
{
	if [ -e "${VALIDATION_PID_FILE}" ]; then
		rm -f ${VALIDATION_PID_FILE}
	else
		logToSys "Validation has stopped already"
		exit 0
	fi
}

autoValidation()
{
	tmpTaskCfg="/mnt/ramdisk/validation_task.xml"
	runTaskCfg="/etc/conf.d/event/task/validation_task.xml"

	sync

	[ ! -e "${VALIDATION_CFG}" ] && exit 0

	if [ "$(getXmlItemVal /validation/enable)" = "1" ]; then
		weekday="$(getXmlItemVal /validation/duration/weekday)"

		time="$(getXmlItemVal /validation/duration/time)"
		if [ "${time}" = "00:00-24:00" ]; then
			time="00:00:00-23:59:59"
		else
			beginTime="$(echo "${time}" | cut -d '-' -f 1):00"
			endTime="$(echo "${time}" | cut -d '-' -f 2):00"
			time="${beginTime}-${endTime}"
		fi

		script_path="${BASEDIR}/validation.sh"

		printf "${taskPattern}" "${weekday}" "${time}" "${script_path}" "${script_path}" > "${tmpTaskCfg}"

		mv "${tmpTaskCfg}" "${runTaskCfg}"
	else
		if [ -e "${runTaskCfg}" ]; then
			rm -f "${runTaskCfg}"
			stopValidation
		fi
	fi

	sync

	"${EVENT_MGR}" reload
}

case "${ACTION}" in
	"start")
		startValidation
		;;
	"stop")
		stopValidation
		;;
	"auto")
		autoValidation
		;;
	*)
		logToSys "Unknown action (${ACTION}) for controlling validation"
		exit 1
		;;
esac

